--------------------------------------------------------------------------
-- Nox-MoTL.lua 
--------------------------------------------------------------------------
--[[
MoTL
Author: Nox US-Ragnaros

Change Log:
	v1.0 
		- Initial Release
	v1.1
		- Bug fixes
		- Added warning text
	v1.3
		- Fixed Curse issue
		- Added loading message
	v1.4
		- Curse Test
]]--

if (MarkOfTheLegionHelper) then
	-- Do nothing
else
	MarkOfTheLegionHelper = { };
end

MarkOfTheLegionHelperOptions = { };

MarkOfTheLegionHelper = {
	Version = "1.3";
	ModCode = "MotLH";
	Setting = { };
	Data = {
		Me = UnitName("player");
		Realm = gsub(GetRealmName(), "%s", "");
		MeFull = UnitName("player").."-"..gsub(GetRealmName(), "%s", "");
		DebuffedList = { };
		TankSpots = { };
	};
	
	Sounds = {
		["green"] = "Interface\\AddOns\\Nox-MoTL\\Sounds\\green.ogg",
		["orange"] = "Interface\\AddOns\\Nox-MoTL\\Sounds\\orange.ogg",
		["purple"] = "Interface\\AddOns\\Nox-MoTL\\Sounds\\purple.ogg",
		["yellow"] = "Interface\\AddOns\\Nox-MoTL\\Sounds\\yellow.ogg",
	};	
}

function MarkOfTheLegionHelper.OnEvent(self, event, ...)
	if (event == "VARIABLES_LOADED") then
		RegisterAddonMessagePrefix(MarkOfTheLegionHelper.ModCode);
		MarkOfTheLegionHelperTooltip:SetOwner(UIParent, "ANCHOR_NONE");
		MarkOfTheLegionHelperFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		MarkOfTheLegionHelperFrame:RegisterEvent("CHAT_MSG_ADDON");
		MarkOfTheLegionHelperFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
		MarkOfTheLegionHelperFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
		MarkOfTheLegionHelper.ChatPrint("v"..MarkOfTheLegionHelper.Version.." cargado con éxito.")
		return;
	end
	if (event == "PLAYER_REGEN_DISABLED") then
		MarkOfTheLegionHelper.Data.DebuffedList = { };
		return;
	end
	if (event == "GROUP_ROSTER_UPDATE") then
		MarkOfTheLegionHelper.Data.TankSpots = { };
		if (UnitExists("raid1")) then
			for i = 1, GetNumGroupMembers(), 1 do
				local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML, combatRole = GetRaidRosterInfo(i);
				if (subgroup < 5 and (combatRole == "TANK" or role == "MAINTANK")) then
					tinsert(MarkOfTheLegionHelper.Data.TankSpots, i);
				end
			end
		end
	end
	if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		if UnitIsGroupLeader("player") then
			local timestamp, eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, misc1, misc2, misc3, misc4, misc5, misc6, misc7 = ...; 
			if (tostring(eventType) == "SPELL_AURA_APPLIED" and tonumber(misc1) == 187050) then
				local unit = MarkOfTheLegionHelper.FindRaidUnit(destName);
				if (unit) then
					tinsert(MarkOfTheLegionHelper.Data.DebuffedList, unit);
					if (#MarkOfTheLegionHelper.Data.DebuffedList == 4) then
						MarkOfTheLegionHelper.Execute(MarkOfTheLegionHelper.Data.DebuffedList);
					end
				end
			end
		end
		return;
	end
	if (event == "CHAT_MSG_ADDON") then
		local prefix, message, messageType, sender = ...;
		if (prefix == MarkOfTheLegionHelper.ModCode and message and sender) then
			MarkOfTheLegionHelper.ReceiveComm(message, sender);
		end
		return;
	end
end

function MarkOfTheLegionHelper.ReceiveComm(sMessage, sSender)
	MarkOfTheLegionHelper.DebugPrint(sMessage)
	local order = MarkOfTheLegionHelper.Split(sMessage, ":");
	if (order[1] == "O" and order[2] and UnitExists("raid1")) then
		local unit = MarkOfTheLegionHelper.FindRaidUnit(sSender);
		if unit and UnitIsGroupLeader("raid"..unit) then
			-- Only read orders if it came from the leader
			MarkOfTheLegionHelper.ReadOrders(order[2]);
		end
	end
end


function MarkOfTheLegionHelper.Execute(tList)
	if not (tList) then
		return;
	end
	MarkOfTheLegionHelper.Data.DebuffedList = { };
	local orders = "";
	local ordersDebug = "";
	local group = {
		[1] = { },	
		[2] = { },
		[3] = { },
		[4] = { },
	};
	local usedUnits = { };

	-- Assign debuffed tanks to spots
	for x, unitNumber in pairs(tList) do 
		if tContains(MarkOfTheLegionHelper.Data.TankSpots, unitNumber) and tContains(tList, unitNumber) then
			MarkOfTheLegionHelper.SetGroupMarker("raid"..unitNumber, (#usedUnits + 1));
			MarkOfTheLegionHelper.DebugPrint("Assigning debuffed tank "..UnitName("raid"..unitNumber).." to group "..(#usedUnits + 1));
			tinsert(group[(#usedUnits + 1)], unitNumber);
			tinsert(usedUnits, unitNumber);
		end
	end
	
	-- Split debuffed players in each spot
	for x, unitNumber in pairs(tList) do 
		if not tContains(usedUnits, unitNumber) then
			MarkOfTheLegionHelper.SetGroupMarker("raid"..unitNumber, (#usedUnits + 1));
			MarkOfTheLegionHelper.DebugPrint("Assigning debuffed player "..UnitName("raid"..unitNumber).." to group "..(#usedUnits + 1));
			tinsert(group[(#usedUnits + 1)], unitNumber);
			tinsert(usedUnits, unitNumber);
		end
	end

	-- Assign tanks to spots
	for i, unitNumber in pairs(MarkOfTheLegionHelper.Data.TankSpots) do 
		if not tContains(usedUnits, unitNumber) then
			MarkOfTheLegionHelper.DebugPrint("Assigning tank "..UnitName("raid"..unitNumber).." to group "..((i % 2) + 1));
			tinsert(group[((i % 2) + 1)], unitNumber);
			tinsert(usedUnits, unitNumber);
		end
	end

	-- Fill remaining spots
	for i = 1, GetNumGroupMembers(), 1 do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML, combatRole = GetRaidRosterInfo(i);
		if (subgroup < 5) then
			if not tContains(usedUnits, i) then
				local groupAssignment = 1;
				if (#group[1] > 4) then
					groupAssignment = 2;
				end
				if (#group[2] > 4) then
					groupAssignment = 3;
				end
				if (#group[3] > 4) then
					groupAssignment = 4;
				end
				tinsert(group[groupAssignment], i);
				tinsert(usedUnits, i);
			end
		else
			MarkOfTheLegionHelper.DebugPrint("Ignoring "..UnitName("raid"..i).." in group "..subgroup);
		end
	end
	
	for i, g in pairs(group) do 
		ordersDebug = MarkOfTheLegionHelper.PrintGroupMarker(i)..": ";
		for j, unitNumber in pairs(g) do 
			local unit = unitNumber;
			if (unitNumber < 10) then
				unit = "0"..unitNumber;
			end
			orders = orders..unit.."/"..i..",";
			ordersDebug = ordersDebug..UnitName("raid"..unitNumber)..", ";
		end
		MarkOfTheLegionHelper.ChatPrint(ordersDebug, .5, .5, .5);
	end
	MarkOfTheLegionHelper.ChatPrint("-----------------------------------", .5, .5, .5);
	
	SendAddonMessage(MarkOfTheLegionHelper.ModCode,"O:"..orders,"RAID");
end

-- Orders format: "[raid ID #1]/[group #],[raid ID #2]/[group #],..."
function MarkOfTheLegionHelper.ReadOrders(sOrders)
	local unitNumber = MarkOfTheLegionHelper.FindPlayer();
	if (unitNumber < 10) then
		unitNumber = "0"..unitNumber;
	end
	local textList = MarkOfTheLegionHelper.Split(sOrders,",");
	for i, text in pairs(textList) do 
		if (string.find(text,unitNumber.."/",1)) then
			local groupNumber = string.gsub(text, unitNumber.."/", "");
			groupNumber = tonumber(groupNumber);
			MarkOfTheLegionHelper.AnnounceGroup(groupNumber);
			return;
		end
	end
end

function MarkOfTheLegionHelper.AnnounceGroup(iGroup)
	if (iGroup == 1) then
		-- Yellow
		MarkOfTheLegionHelper.ChatPrint(MarkOfTheLegionHelper.PrintGroupMarker(1).." >>> CORRE A AMARILLO <<< "..MarkOfTheLegionHelper.PrintGroupMarker(1), 1, 1, 0)
		MarkOfTheLegionHelper.WarningPrint(MarkOfTheLegionHelper.PrintGroupMarker(1).." >>> CORRE A AMARILLO <<< "..MarkOfTheLegionHelper.PrintGroupMarker(1))
		MarkOfTheLegionHelper.PlaySound("yellow");
	elseif (iGroup == 2) then
		-- Orange
		MarkOfTheLegionHelper.ChatPrint(MarkOfTheLegionHelper.PrintGroupMarker(2).." >>> CORRE A NARANJA <<< "..MarkOfTheLegionHelper.PrintGroupMarker(2), 1, .5, 0)
		MarkOfTheLegionHelper.WarningPrint(MarkOfTheLegionHelper.PrintGroupMarker(2).." >>> CORRE A NARANJA <<< "..MarkOfTheLegionHelper.PrintGroupMarker(2))
		MarkOfTheLegionHelper.PlaySound("orange");
	elseif (iGroup == 3) then
		-- Purple
		MarkOfTheLegionHelper.ChatPrint(MarkOfTheLegionHelper.PrintGroupMarker(3).." >>> CORRE A MORADO <<< "..MarkOfTheLegionHelper.PrintGroupMarker(3), 1, 0, 1)
		MarkOfTheLegionHelper.WarningPrint(MarkOfTheLegionHelper.PrintGroupMarker(3).." >>> CORRE A MORADO <<< "..MarkOfTheLegionHelper.PrintGroupMarker(3))
		MarkOfTheLegionHelper.PlaySound("purple");
	elseif (iGroup == 4) then
		-- Green
		MarkOfTheLegionHelper.ChatPrint(MarkOfTheLegionHelper.PrintGroupMarker(4).." >>> CORRE A VERDE <<< "..MarkOfTheLegionHelper.PrintGroupMarker(4), 0, 1, 0)
		MarkOfTheLegionHelper.WarningPrint(MarkOfTheLegionHelper.PrintGroupMarker(4).." >>> CORRE A VERDE <<< "..MarkOfTheLegionHelper.PrintGroupMarker(4))
		MarkOfTheLegionHelper.PlaySound("green");
	end		
end

function MarkOfTheLegionHelper.PrintGroupMarker(iMark)
	return "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_"..iMark..":16\124t";
end

function MarkOfTheLegionHelper.SetGroupMarker(sUnit, iGroup)
	if UnitExists(sUnit) then
		if (iGroup == 1) then
			-- Yellow
			SetRaidTarget(sUnit, 1);
		elseif (iGroup == 2) then
			-- Orange
			SetRaidTarget(sUnit, 2);
		elseif (iGroup == 3) then
			-- Purple
			SetRaidTarget(sUnit, 3);
		elseif (iGroup == 4) then
			-- Green
			SetRaidTarget(sUnit, 4);
		end	
	end
end

function MarkOfTheLegionHelper.FindPlayer()
	for i = 1, GetNumGroupMembers(), 1 do
		if (UnitIsUnit("player", "raid"..i)) then
			return i;
		end
	end
	return 0;
end

function MarkOfTheLegionHelper.ChatPrint(str, color1, color2, color3)
	DEFAULT_CHAT_FRAME:AddMessage("[Nox-MoTL] "..(str or ""), color1 or 0.25, color2 or 1.0, color3 or 0.25);
end

function MarkOfTheLegionHelper.DebugPrint(str)
	--DEFAULT_CHAT_FRAME:AddMessage("[Legion Mark] "..(str or ""), color1 or 1.0, color2 or .25, color3 or 0.25);
end

function MarkOfTheLegionHelper.WarningPrint(str)
	RaidNotice_AddMessage(RaidWarningFrame, (str or ""), ChatTypeInfo["RAID_WARNING"])
end

function MarkOfTheLegionHelper.OnLoad()
	MarkOfTheLegionHelperFrame:RegisterEvent("VARIABLES_LOADED");
end

function MarkOfTheLegionHelper.FullName(sUnit, bForceRealm)
	local name, realm = UnitName(sUnit);
	if (realm and realm ~= "") then
		return name.."-"..realm;
	elseif (name and bForceRealm) then
		return name.."-"..MarkOfTheLegionHelper.Data.Realm;
	else
		return name;
	end
end

function MarkOfTheLegionHelper.PlaySound(sColor)
	local sound = MarkOfTheLegionHelper.Sounds[sColor or ""];
	if (sound) then
		PlaySoundFile(sound, "Master");
	end
end

function MarkOfTheLegionHelper.Split(str, delim, maxNb)
    if string.find(str, delim) == nil then
        return { str }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gmatch(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
		for i = #result, 1, -1 do
			if (not result[i] or result[i] == "") then
				table.remove(result, i);
			end
		end
    return result
end

function MarkOfTheLegionHelper.FindRaidUnit(sName)
	local raidmembers = GetNumGroupMembers();
	if (raidmembers > 0) then
		for i = 1, raidmembers, 1 do
			if (MarkOfTheLegionHelper.CompareName(sName, "raid"..i)) then
				return i;
			end
		end
	end
	return nil;
end	

function MarkOfTheLegionHelper.CompareName(sName, sUnit)
	return (sName == MarkOfTheLegionHelper.FullName(sUnit) or sName == MarkOfTheLegionHelper.FullName(sUnit, true));
end

function MarkOfTheLegionHelper.FullName(sUnit, bForceRealm)
	local name, realm = UnitName(sUnit);
	if (realm and realm ~= "") then
		return name.."-"..realm;
	elseif (name and bForceRealm) then
		return name.."-"..MarkOfTheLegionHelper.Data.Realm;
	else
		return name;
	end
end
